#!/usr/bin/perl
use strict;
use DBI;
use Sys::Hostname;
use Term::ReadKey;
use OME::Session;
use OME::SessionManager;


my $manager = OME::SessionManager->new();
my $session = $manager->TTYlogin();
my $factory = $session->Factory();


print "\nCreating OME user...\n";
print "I will now ask you for a username and password for the new OME user\n\n";

my @login = getpwuid($<);
my $omeName = $login[0];
my $password = $login[1];
my $personalInfo = $login[6];
my $dataDir = $login[7];
my ($fullName, $rest) = split (',',$personalInfo,2);
my ($firstName,$lastName) = split (' ',$fullName,2);
my $email = $omeName.'@'.hostname;

print "Full name (first last) [$firstName $lastName]: ";
my $personalInfoIn = <STDIN>;
chomp($personalInfoIn);
if ($personalInfoIn) {
	$personalInfo = $personalInfoIn if $personalInfoIn;
	($fullName, $rest) = split (',',$personalInfo,2);
	($firstName,$lastName) = split (' ',$fullName,2);
}
	
ReadMode(1);
print "Username [$omeName]: ";
my $omeInput = <STDIN>;
chomp($omeInput);
$omeName = $omeInput if $omeInput;

ReadMode(2);
my $password;
print "Password: ";
my $passwordInput = ReadLine(0);
print "\n";
chomp($passwordInput);

ReadMode(1);
if ($passwordInput) {
    $password = $passwordInput;
    my $salt = join('',('.','/',0..9,'A'..'Z','a'..'z')[rand 64, rand 64]);
    $password = crypt($passwordInput,$salt);
}

print "email [$email]: ";
my $emailInput = <STDIN>;
chomp($emailInput);
$email = $emailInput if $emailInput;


print "Data directory [$dataDir]: ";
my $dataDirInput = <STDIN>;
chomp ($dataDirInput);
$dataDir = $dataDirInput if $dataDirInput;
if (not -e $dataDir) {
	print "Directory '$dataDir' does not exist.  Create it? [No]: ";
	my $YorN = <STDIN>;
	$YorN = uc (chomp($YorN));
	if ($YorN && ($YorN eq 'Y' || $YorN eq 'YES')){
		die "Couldn't make directory $dataDir: $!\n" if system ( "mkdir $dataDir");
		chmod 0755, $dataDir;
	}
}


my @groups = $factory->findAttributes('Group');

printf ("%9s| %s\n",'GroupID','Leader');
printf ("%9s+ %s\n",'-------','------');

my %groupIDs;
my $groupIDdef;

foreach my $group (@groups) {
    my $groupID = $group->id();
    my $leader = $group->Leader();
    my $expID = $leader->id();
    my $gFirstName = $leader->FirstName();
    my $gLastName = $leader->LastName();

    printf ("%9d| %s\n",$groupID,"$gFirstName $gLastName ($expID)");
    $groupIDs{$groupID} = $group;
    $groupIDdef = $groupID unless defined $groupIDdef;
}

my $groupID = $groupIDdef;
print "Select the Group ID that this Experimenter belongs to: [$groupIDdef]: ";
my $groupIDinput = <STDIN>;
chomp($groupIDinput);
$groupID = $groupIDinput if $groupIDs{$groupIDinput};
print "\n\n";

# The Experimenter attribute type does not contain semantic elements
# for OME name or password, so we must still add the user via a 
# hard-coded SQL statement.

my $sql = <<ENDSQL;
INSERT INTO EXPERIMENTERS (ATTRIBUTE_ID,MODULE_EXECUTION_ID,OME_NAME,FIRSTNAME,
                           LASTNAME,EMAIL,GROUP_ID,PASSWORD,DATA_DIR) VALUES (
	nextval ('ATTRIBUTE_SEQ'),
        null,
	'$omeName',
	'$firstName',
	'$lastName',
	'$email',
	$groupID,
	'$password',
	'$dataDir'
);
ENDSQL

print "$sql\n";

my $dbh = $session->DBH();
$dbh->do ($sql );
$dbh->commit();

print "\nDone.  New user added.\n\n";

