#!/usr/bin/perl -w
#
# sample-depopulate.pl
# Copyright (C) 2003 Open Microscopy Environment, MIT
# Author:  Brian S. Hughes
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    This script will remove all the OME database entries added by
#    the sample-populate.pl script.

use strict;
use DBI;
use DBD::Pg;

my $dbh;
my $sth;
my $ndx;
my $statement;
my @dels = ('project_dataset_map', '', '',
	    'datasets', 'dataset_id', '0',
	    'projects', 'project_id', 0,
	    'experimenters', 'experimenter_id', 1,
	    'groups', 'group_id', 1);

my $admin = set_to_admin();
$dbh = DBI->connect("dbi:Pg:dbname=ome", $admin, '', {RaiseError=>1});

# delete all the entries in the listed tables that have an ID greater
# than the listed ID.
$ndx = scalar @dels;
for (my $i = 0; $i < $ndx; $i += 3) {
    if ($dels[$i+1] ne '') {
	$statement = "DELETE FROM $dels[$i] WHERE $dels[$i+1] > $dels[$i+2]";
    } else {
	$statement = "DELETE FROM $dels[$i]";
    }
    $dbh->do($statement);
}

$dbh->disconnect;


# Ask user for DB admin username
sub set_to_admin {
    my $adminUser = "";
    my $user = get_uid();
    print "OME administrative user [$user]: ";
    my $adminUserIn = <STDIN>;
    chomp $adminUserIn;
    if ($adminUserIn) {
        $adminUser = $adminUserIn;
    } else {
	$adminUser = $user;
    }
    return $user;
}


# find out current username
sub get_uid {
my @login = getpwuid($>);
	return ( $login[0] );
}

