-- Author:  Daniel R. Rines (drrines@mit.edu)
-- Copyright 1999-2001 Daniel R. Rines
-- This file is part of OME.
-- 
--     OME is free software; you can redistribute it and/or modify
--     it under the terms of the GNU General Public License as published by
--     the Free Software Foundation; either version 2 of the License, or
--     (at your option) any later version.
-- 
--     OME is distributed in the hope that it will be useful,
--     but WITHOUT ANY WARRANTY; without even the implied warranty of
--     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--     GNU General Public License for more details.
-- 
--     You should have received a copy of the GNU General Public License
--     along with OME; if not, write to the Free Software
--     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- 
--

CREATE TABLE DISTANCE_TO_SPINDLE_AXIS (
ATTRIBUTE_OF  OID,
ANALYSIS_ID   OID        REFERENCES ANALYSES (ANALYSIS_ID) DEFERRABLE INITIALLY DEFERRED,
DISTANCE      FLOAT,
LENGTH        FLOAT
);
CREATE INDEX DISTANCE_TO_SPINDLE_AXIS_ATTRIBUTE_OF_idx ON DISTANCE_TO_SPINDLE_AXIS (ATTRIBUTE_OF);
CREATE INDEX DISTANCE_TO_SPINDLE_AXIS_ANALYSIS_ID_idx  ON DISTANCE_TO_SPINDLE_AXIS (ANALYSIS_ID);

CREATE TABLE INPUT_FINDSPINDLES (
ANALYSIS_ID    OID    REFERENCES ANALYSES DEFERRABLE INITIALLY DEFERRED,
DIST_CUTOFF  FLOAT,
SIGMA_CUTOFF FLOAT
);
--
INSERT INTO PROGRAMS (PROGRAM_ID,PROGRAM_NAME,LOCATION,DESCRIPTION,CATEGORY,INPUT_TABLE) VALUES (
nextval ('PROGRAM_SEQ'),
'findSpindles',
'/OME/bin/findSpindles/',
'Clusters segmented features into spindles as labeled with 3 gfp markers - 2 on SPD and 1 on Cen.',
'Chromosome Dynamics',
'INPUT_FINDSPINDLES'
);
--
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'CLUSTER_F');
--
--
CREATE TABLE SPINDLE_DYNAMICS (
ATTRIBUTE_OF     OID,
ANALYSIS_ID      OID        REFERENCES ANALYSES (ANALYSIS_ID) DEFERRABLE INITIALLY DEFERRED,
TIME_PT          INTEGER,
ELAPSED_TIME     FLOAT,
D_ALPHA          FLOAT,
D_BETA           FLOAT,
D_GAMMA          FLOAT,
D_EPSILON        FLOAT,
D_ALPHABETA      FLOAT,
L_ALPHA          FLOAT,
L_BETA           FLOAT,
L_GAMMA          FLOAT,
L_EPSILON        FLOAT,
L_AB             FLOAT,
RHO_ALPHA        FLOAT,
RHO_BETA         FLOAT,
RHO_GAMMA        FLOAT,
RHO_EPSILON      FLOAT,
THETA_ALPHA      FLOAT,
THETA_BETA       FLOAT,
THETA_GAMMA      FLOAT,
THETA_EPSILON    FLOAT,
PHI_ALPHA        FLOAT,
PHI_BETA         FLOAT,
DELTA_AB         FLOAT,
DELTA_ALPHA      FLOAT,
DELTA_BETA       FLOAT,
DELTA_GAMMA      FLOAT,
DELTA_RHO_ALPHA  FLOAT,
DELTA_RHO_BETA   FLOAT,
DELTA_ALPHABETA  FLOAT,
DELTA_A          FLOAT,
DELTA_B          FLOAT,
NU_AB            FLOAT,
NU_ALPHA         FLOAT,
NU_BETA          FLOAT,
NU_GAMMA         FLOAT,
NU_RHO_ALPHA     FLOAT,
NU_RHO_BETA      FLOAT,
NU_ALPHABETA     FLOAT,
NU_A             FLOAT,
NU_B             FLOAT
);
CREATE INDEX SPINDLE_DYNAMICS_ATTRIBUTE_OF_idx ON SPINDLE_DYNAMICS (ATTRIBUTE_OF);
CREATE INDEX SPINDLE_DYNAMICS_ANALYSIS_ID_idx  ON SPINDLE_DYNAMICS (ANALYSIS_ID);
--
--
--
CREATE TABLE SPINDLE_SPOT_CLASS (
ATTRIBUTE_OF  OID,
ANALYSIS_ID   OID        REFERENCES ANALYSES (ANALYSIS_ID) DEFERRABLE INITIALLY DEFERRED,
CLASS         VARCHAR(20)
);
CREATE INDEX SPOT_CLASS_ATTRIBUTE_OF_idx ON SPINDLE_SPOT_CLASS (ATTRIBUTE_OF);
CREATE INDEX SPOT_CLASS_ANALYSIS_ID_idx  ON SPINDLE_SPOT_CLASS (ANALYSIS_ID);
--
--
--
CREATE TABLE INPUT_MANSPBSPOTS (
ANALYSIS_ID    OID    REFERENCES ANALYSES DEFERRABLE INITIALLY DEFERRED
);
--
INSERT INTO PROGRAMS (PROGRAM_ID,PROGRAM_NAME,LOCATION,DESCRIPTION,CATEGORY,INPUT_TABLE) VALUES (
nextval ('PROGRAM_SEQ'),
'manSPBSpots',
'/var/www/cgi-bin/sorgerlab/',
'Manual spot entry for SPB spots - reports distance to spindle axis, position, and cluster ID',
'Chromosome Dynamics',
'INPUT_MANSPBSPOTS'
);
--
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'LOCATION');
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'TIMEPOINT');
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'CLUSTER_F');
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'SIGNAL');
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'DISTANCE_TO_SPINDLE_AXIS');
--
--
--
CREATE TABLE INPUT_CLICKYSPOTS (
ANALYSIS_ID    OID    REFERENCES ANALYSES DEFERRABLE INITIALLY DEFERRED,
FILETYPE       VARCHAR(20),
ORIENTATION    VARCHAR (20)
);
--
INSERT INTO PROGRAMS (PROGRAM_ID,PROGRAM_NAME,LOCATION,DESCRIPTION,CATEGORY,INPUT_TABLE) VALUES (
nextval ('PROGRAM_SEQ'),
'ClickySpots',
'/var/www/cgi-bin/perl/',
'Loads auxiliary file containing manualy-determined spot coordinates and spindle/chromosome assignments',
'Chromosome Dynamics',
'INPUT_CLICKYSPOTS'
);
--
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'LOCATION');
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'SPINDLE_SPOT_CLASS');
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'TIMEPOINT');
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'SPINDLE_DYNAMICS');
INSERT INTO ATTRIBUTE_LIST (PROGRAM_ID,TABLE_NAME) VALUES (currval('PROGRAM_SEQ'),'SIGNAL');


