/*
 * org.openmicroscopy.CategorizedModules
 *
 * Copyright (C) 2002 Open Microscopy Environment, MIT
 * Author:  Douglas Creager <dcreager@alum.mit.edu>
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openmicroscopy;

import java.util.SortedMap;
import java.util.TreeMap;
import java.util.SortedSet;
import java.util.TreeSet;

public class CategorizedModules
{
    protected static SortedMap  categories = new TreeMap();

    public static SortedMap getCategories()
    { return categories; }
    public static SortedSet getCategory(String category)
    { return (SortedSet) categories.get(category); }

    public static void addModule(Module module)
    {
        String category = module.getCategory();
        SortedSet categoryList = (SortedSet) categories.get(category);
        if (categoryList == null)
        {
            categoryList = new TreeSet();
            categories.put(category,categoryList);
        }
        categoryList.add(module);
    }

    public static void removeModule(Module module)
    {
        SortedSet  categoryList = (SortedSet) categories.get(module.getCategory());
        if (categoryList != null)
            categoryList.remove(module);
    }
}
