/*
 * org.openmicroscopy.remote.RemoteImage
 *
 * Copyright (C) 2002 Open Microscopy Environment, MIT
 * Author:  Douglas Creager <dcreager@alum.mit.edu>
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openmicroscopy.remote;

import org.openmicroscopy.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

public class RemoteImage
    extends RemoteOMEObject
    implements Image
{
    static
    {
        RemoteObject.addClass("OME::Image",RemoteImage.class);
        RemoteObject.addClass("OME::Image::DatasetMap",DatasetLink.class);
    }
        
    public RemoteImage() { super(); }
    public RemoteImage(String reference) { super(reference); }

    public String getName()
    { return getStringElement("name"); }
    public void setName(String name)
    { setStringElement("name",name); }

    public String getDescription()
    { return getStringElement("description"); }
    public void setDescription(String description)
    { setStringElement("description",description); }

    public String getImageGUID()
    { return getStringElement("image_guid"); }
    public void setImageGUID(String imageGUID)
    { setStringElement("image_guid",imageGUID); }

    public String getCreated()
    { return getStringElement("created"); }
    public void setCreated(String created)
    { setStringElement("created",created); }

    public String getInserted()
    { return getStringElement("inserted"); }
    public void setInserted(String inserted)
    { setStringElement("inserted",inserted); }

    public Attribute getExperimenter()
    { return getAttributeElement("experimenter"); }
    public void setExperimenter(Attribute experimenter)
    { setAttributeElement("experimenter",experimenter); }

    public Attribute getGroup()
    { return getAttributeElement("group"); }
    public void setGroup(Attribute group)
    { setAttributeElement("group",group); }

    public List getDatasets()
    {
        List linkList = getRemoteListElement(DatasetLink.class,
                                             "dataset_links");
        List datasetList = new ArrayList();
        Iterator i = linkList.iterator();
        while (i.hasNext())
        {
            DatasetLink link = (DatasetLink) i.next();
            datasetList.add(link.getDataset());
        }
        return datasetList;
    }

    public Iterator iterateDatasets()
    {
        final RemoteIterator i = (RemoteIterator) 
            getRemoteElement(RemoteIterator.class,
                             "iterate_dataset_links");
        i.setClass(DatasetLink.class);
        return new Iterator()
            {
                public boolean hasNext() { return i.hasNext(); }
                public void remove() { i.remove(); }
                public Object next()
                {
                    DatasetLink link = (DatasetLink) i.next();
                    return link.getDataset();
                }
            };
    }

    public List getAllFeatures()
    { return getRemoteListElement(RemoteFeature.class,"all_features"); }

    public Iterator iterateAllFeatures()
    {
        RemoteIterator i = (RemoteIterator)
            getRemoteElement(RemoteIterator.class,
                             "iterate_all_features");
        i.setClass(RemoteFeature.class);
        return i;
    }

    public ImagePixels getPixels(Attribute pixels, Attribute dimensions)
    {
        Attribute repository = pixels.getAttributeElement("Repository");
        if (LocalImagePixels.isRepositoryLocal(repository))
        {
            LocalImagePixels pix = new LocalImagePixels(pixels,dimensions);
            return pix;
        } else {
            Object o = caller.dispatch("OME::Image","GetPix",
                                       new Object[] { pixels, dimensions });
            if (o == null) return null;
            RemoteImagePixels pix = (RemoteImagePixels)
                instantiate(RemoteImagePixels.class,(String) o);
            pix.setDimensions(dimensions);
            return pix;
        }
    }

    static class DatasetLink
        extends RemoteOMEObject
    {
        public DatasetLink() { super(); }
        public DatasetLink(String reference) { super(reference); }

        Image getImage()
        { return (Image) getRemoteElement(RemoteImage.class,"image"); }

        Dataset getDataset()
        { return (Dataset) getRemoteElement(RemoteDataset.class,"dataset"); }
    }
}
