/*
 * org.openmicroscopy.remote.RemoteLookupTable
 *
 * Copyright (C) 2002 Open Microscopy Environment, MIT
 * Author:  Douglas Creager <dcreager@alum.mit.edu>
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openmicroscopy.remote;

import org.openmicroscopy.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

public class RemoteLookupTable
    extends RemoteOMEObject
    implements LookupTable
{
    static
    {
        RemoteObject.addClass("OME::LookupTable",RemoteLookupTable.class);
        RemoteObject.addClass("OME::LookupTable::Entry",
                              RemoteLookupTable.Entry.class);
    }

    public RemoteLookupTable() { super(); }
    public RemoteLookupTable(String reference) { super(reference); }

    public String getName()
    { return getStringElement("name"); }
    public void setName(String name)
    { setStringElement("name",name); }

    public String getDescription()
    { return getStringElement("description"); }
    public void setDescription(String description)
    { setStringElement("description",description); }

    public List getEntries()
    { return getRemoteListElement(Entry.class,"all_features"); }

    public Iterator iterateEntries()
    {
        RemoteIterator i = (RemoteIterator)
            getRemoteElement(RemoteIterator.class,
                             "iterate_all_features");
        i.setClass(Entry.class);
        return i;
    }

    public static class Entry
        extends RemoteOMEObject
        implements LookupTable.Entry
    {
        public Entry() { super(); }
        public Entry(String reference) { super(reference); }

        public LookupTable getLookupTable()
        { return (LookupTable) getRemoteElement(RemoteLookupTable.Entry.class,
                                                "lookup_table"); }

        public String getValue()
        { return getStringElement("value"); }
        public void setValue(String value)
        { setStringElement("value",value); }

        public String getLabel()
        { return getStringElement("label"); }
        public void setLabel(String label)
        { setStringElement("label",label); }
    }
}
