#!/usr/bin/perl -w
#
# Exporter.pm
# Copyright (C) 2002 Open Microscopy Environment, MIT
# Author:  Brian S. Hughes
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# OME's image export class. It creates an instance of the Export_writer
# class for each image and has that instance's methods and sub-classes
# do the actual import work.
#

# ---- Public routines -------
# new()

package OME::ImportExport::Exporter;
use strict;
use OME::ImportExport::Export_writer;
use Carp;
use File::Basename;
use vars qw($VERSION);
$VERSION = 2.000_000;

sub new {
    my $export_writer;
    my $image_list_ref;
    my $export_type;
    my $session;
    my $repository;
    my $ky;
    my $status = "";

    my $self = {};

    my $invoker = shift;
    my $class = ref($invoker) || $invoker;   # called from class or instance

    $session = shift;

    $export_type = shift;
    $status = "No export type given"
	unless $export_type;

    $image_list_ref = shift;         # reference list of export images
    $status = "No image file to export"
	unless $image_list_ref;

    $repository = shift;
    $status = "No repository found"
	unless $repository;

    return $status
	unless $status eq "";
    
    $export_writer = new OME::ImportExport::Export_writer($session, $export_type, $image_list_ref, $repository, $self);
    $status = "Couldn\'t make Export_writer instance";
    if (defined $export_writer) {
	$status = $export_writer->export;
    }

    if ($status ne "") {
	carp $status;
    }


}


1;
