# OME/Web/GetInfo.pm

# Copyright (C) 2002 Open Microscopy Environment, MIT
# Author:  Jean-Marie Burel <j.burel@dundee.ac.uk>
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package OME::Web::GetInfo;

use strict;
use vars qw($VERSION);
$VERSION = 2.000_000;
use CGI;
use OME::Tasks::ProjectManager;
use OME::Tasks::DatasetManager;
use OME::Web::Helper::HTMLFormat;
use OME::Web::Helper::JScriptFormat;

use base qw(OME::Web);


sub getPageTitle {
	return "Open Microscopy Environment" ;
}

sub getPageBody{

	my $self=shift;
	my $cgi=$self->CGI();
	my $session=$self->Session();
	my $body="";
	my $projectManager=new OME::Tasks::ProjectManager($session);
	my $datasetManager=new OME::Tasks::DatasetManager($session);
	my $htmlFormat=new OME::Web::Helper::HTMLFormat;
	my $jscriptFormat=new OME::Web::Helper::JScriptFormat;

	my $projectID=$cgi->url_param('ProjectID');
	my $datasetID=$cgi->url_param('DatasetID');
	my $bool=$cgi->url_param('Bool');

  if ( !defined $projectID and !defined $datasetID){
   die "Package needs ProjectID as url parameters. Neither was supplied. This message generated by ".(ref $self)."->createOMEPage().";
	
  }
  if (defined $projectID){
    my $project=$projectManager->load($projectID);
    $body.=getprojectinfo($project,$htmlFormat);
  }
  if (defined $datasetID){
	
    my $dataset=$datasetManager->load($datasetID);
    $body.="<h4>Import succesful</h4>" if defined $bool;
    $body.=getdatasetinfo($dataset,$htmlFormat);
  }
   $body.=$jscriptFormat->popUpDataset();
   $body.=$jscriptFormat->popUpImage();
   $body.=$jscriptFormat->closeButton();

   return('HTML',$body);




}

#--------------------
#PRIVATE METHODS
#-------------------

sub getprojectinfo{
  my ($project,$htmlFormat)=@_;
  my $text="";
  my @datasetsused=$project->datasets();
  $text.=$htmlFormat->formatProject($project,1);
  if (scalar(@datasetsused)==0){
	$text.="<b>No dataset associated to this project</b>.<br>";
  }else{
	my %list=map { $_->dataset_id()=>$_} @datasetsused;
	$text.=$htmlFormat->datasetList(\%list);
  }
   return $text;

}


sub getdatasetinfo{
 	my ($dataset,$htmlFormat)=@_;
 	my $text="";
 	my @images=$dataset->images();
  	$text.=$htmlFormat->formatDataset($dataset,1);
	if (scalar(@images)==0){
	   $text.="<b>No image in this dataset.</b>.<br>";
	}else{
	   $text.=$htmlFormat->imageInDataset(\@images);
	}
  	return $text;
}









1;
